/*---------------------------------------------------------------------------*
 *
 *                  cwish - windowing user friendly shell
 *                  -------------------------------------
 *
 *              (c) Copyright Hellmuth Michaelis 1989 - 1993
 *
 *                  Eggerstedtstr. 28
 *                  22765 Hamburg
 *                  Germany
 *
 *                  Tel:    +49 / 40 / 384298    (private)
 *                  Tel:    +49 / 40 / 55903-170 (at work)
 *                  e-mail: hm@hcshh.hcs.de
 *
 *           All rights are reserved except as explicitly granted
 *                  by written permission of the author.
 *
 *             See the file COPYING, distributed with cwish, for
 *                  restriction and warranty information
 *
 *---------------------------------------------------------------------------*
 *
 *	Last Edit-Date: [Wed Oct 20 11:30:36 1993]
 *
 *	-hm	conversion to multiple curses windows
 *	-hm	marked/tagged file sizes
 *
 *----------------------------------------------------------------------------*/

#include "cwish.h"

/*---------------------------------------------------------------------------*
 *	make current file name inverse, fill file attributes
 *---------------------------------------------------------------------------*/
void cur_inv(struct onefile *current)
{
	wmove(file_w,current->orow,current->ocol);

	if(current->tag)
		wattrset(file_w, A_UNDERLINE);
	else
		wattron(file_w, A_REVERSE);

	waddstr(file_w, &(current->onam[1]));
	wattrset(file_w, A_NORMAL);
	attribs(0);
}

/*---------------------------------------------------------------------------*
 *	make current file name blink
 *---------------------------------------------------------------------------*/
void cur_blink(struct onefile *current)
{
	wmove(file_w,current->orow,current->ocol);
	wattrset(file_w, A_BLINK | A_REVERSE);
	waddstr(file_w, &(current->onam[1]));
	wattrset(file_w, A_NORMAL);
}

/*---------------------------------------------------------------------------*
 *	make current filename normal again
 *---------------------------------------------------------------------------*/
void cur_norm(struct onefile *current)
{
	wmove(file_w, current->orow, current->ocol);

	if(current->tag)
		wattrset(file_w, A_REVERSE);
	else
		wattrset(file_w, A_NORMAL);

	waddstr(file_w, &(current->onam[1]));
	wattrset(file_w, A_REVERSE);

}

/*---------------------------------------------------------------------------*
 *	tag current filename and print as tagged
 *---------------------------------------------------------------------------*/
void tag_current(struct onefile *current)
{
	wmove(file_w, current->orow, current->ocol);

	if(current->tag)		/* current filename already tagged ? */
	{
		wattron(file_w, A_REVERSE);
		waddstr(file_w, &(current->onam[1]));
		wattroff(file_w, A_REVERSE);
		current->tag = 0;	/* was tagged, now untag */
		tagsize -= current->osiz;

		if(tagsize == 0)
		{
			wmove(fst_w, 0, COLS-ENTRYSW-ENTRYCW); /* move to entries string */
			wclrtoeol(fst_w);
			wattron(fst_w, A_UNDERLINE);	/* underline on */
		
			if(opt_wildon)	/* wildcarding enabled */
			{
				waddstr(fst_w, "Matches:");	/* description */
			}
			else			/* normal operation */
			{
				waddstr(fst_w, "Entries:");	/* description */
			}
		
			wmove(fst_w, 0, COLS-ENTRYCW);	/* move to entries string */
			wattroff(fst_w, A_UNDERLINE);	/* underline off */
			waddstr(fst_w, counter);	/* print current count */
			return;				/* exit routine ! */
		}
	}
	else
	{
		wattron(file_w, A_UNDERLINE);
		waddstr(file_w, &(current->onam[1]));
		wattroff(file_w, A_UNDERLINE);
		current->tag = 1;	/* was not tagged, now tag */
		if(tagsize == 0)
		{
			wattron(fst_w, A_UNDERLINE);	/* underline on */
			wmove(fst_w, 0, COLS-ENTRYSW-ENTRYCW); /* move to entries string */
			waddstr(fst_w, "Byte:");	/* description */
			wattroff(fst_w, A_UNDERLINE);	/* underline on */		
		}
		tagsize += current->osiz;
	}
	wmove(fst_w, 0, COLS-ENTRYCW-4);/* move to entries string */
	wprintw(fst_w, "%8d", tagsize);	/* print size */
}

/*---------------------------------------------------------------------------*
 *	if filename tagged, print as tagged, else print normal
 *---------------------------------------------------------------------------*/
void is_tagged(struct onefile *current)
{
	wmove(file_w, current->orow, current->ocol);
	
	if(current->tag)		/* current filename tagged ? */
	{
		wattron(file_w, A_REVERSE);
		waddstr(file_w, &(current->onam[1]));
		wattroff(file_w, A_REVERSE);
	}
	else
	{
		waddstr(file_w, &(current->onam[1]));
	}
}

/*---------------------------------------------------------------------------*
 *	untag ALL tagged filenames
 *---------------------------------------------------------------------------*/
void untag_all(void)
{
	struct onefile *cur_ptr;

	int thispage = ((cur_file->page) & PAGE_MSK);
	
	cur_ptr = first;	/* first file in dir */

	tagsize = 0;		/* reset size */

	wmove(fst_w, 0, COLS-ENTRYSW-ENTRYCW); /* move to entries string */
	wclrtoeol(fst_w);
	wattron(fst_w, A_UNDERLINE);	/* underline on */

	if(opt_wildon)	/* wildcarding enabled */
	{
		waddstr(fst_w, "Matches:");	/* description */
	}
	else			/* normal operation */
	{
		waddstr(fst_w, "Entries:");	/* description */
	}

	wmove(fst_w, 0, COLS-ENTRYCW);	/* move to entries string */
	wattroff(fst_w, A_UNDERLINE);	/* underline off */
	waddstr(fst_w, counter);	/* print current count */
	
	while(cur_ptr != NULL)
	{
		if(cur_ptr->tag)
		{
			if((cur_ptr->page == thispage) || (cur_ptr->page == (thispage | FIRST)))
			{
				wmove(file_w, cur_ptr->orow,((cur_ptr->ocol)-1));
				wattroff(file_w,A_REVERSE);
				waddch(file_w,SPACE);
				wattroff(file_w,A_REVERSE);
				waddstr(file_w,&(cur_ptr->onam[1]));
				wattroff(file_w,A_REVERSE);
				waddch(file_w,SPACE);
				wattroff(file_w,A_REVERSE);
			}
		}
		cur_ptr->tag = 0;		
		cur_ptr = cur_ptr->next;
	}
	cur_inv(cur_file);	/* just for shure ... */
}

/*---------------------------------- EOF -------------------------------------*/ 
